const webpack = require('webpack');
const htmlWebpackPlugin = require('html-webpack-plugin');
const { AngularWebpackPlugin } = require('@ngtools/webpack');
//const forkTsCheckerWebpackPlugin = require('fork-ts-checker-webpack-plugin');
const path = require('path');
const pkg = require(path.resolve('./package.json'));

const config = (settings) => ({
    /**
     * Entry points for polyfills and the application
     *
     * Note that polyfills.js is in this package.
     */
    entry: {
        'polyfills': [
            path.join(__dirname, './polyfills.js')
        ],
        'app': [
            'webpack-hot-middleware/client',
            `${settings.rootDir}/${settings.entry}`
        ]
    },
    mode: 'development',
    resolve: {
        /**
         * Set up an aliases for the current package
         */
        alias: {
            [pkg.name]: path.resolve('./src/main')
        },

        extensions: [ '.ts', '.js', '.json' ],

        modules: [ path.resolve('.'), path.resolve('./node_modules') ],
        mainFields: [ 'loader', 'main', 'module' ],
        mainFiles: [ 'index', 'main' ]
    },

    /**
     * Source maps on the cheap
     *
     * See: https://webpack.js.org/configuration/devtool/
     */
    devtool: 'eval-cheap-module-source-map',

    output: {
        /**
         * Set up output paths and file names
         */
        path: path.resolve(settings.rootDir),
        publicPath: settings.baseUrl,

        filename: '[name].bundle.js',
        sourceMapFilename: '[name].map',
        chunkFilename: '[id].chunk.js',

        libraryTarget: 'var',
        library: '[name]_lib'
    },
    resolveLoader: {
        modules: [ path.resolve('./node_modules') ],
        extensions: [ '.ts', '.js', '.json' ],
        mainFields: [ 'loader', 'main', 'module' ]
    },
    module: {
        rules: [
            /**
             * Use the Angular Compiler Webpack plugin
             */
            {
                test: /\.ts$/,
                loader: '@ngtools/webpack',
                exclude: [ /\.(test|spec)\.ts$/ ]
            },

            /**
             * style-loader and css-loader for *.css
             *
             * See: https://github.com/webpack-contrib/style-loader
             * See: https://github.com/webpack-contrib/css-loader
             */
            {
                test: /\.css$/,
                use: [ 'style-loader', 'css-loader' ]
            },

            /**
             * Image assets
             *
             * See: https://webpack.js.org/guides/asset-modules/
             */
            {
                test: /\.(png|jpg|jpe?g|gif|ico)$/,
                type: 'asset/resource'
            },

            /**
             * Font assets
             *
             * See: https://webpack.js.org/guides/asset-modules/
             */
            {
                test: /\.(woff(2)?|ttf|eot|svg)(\?v=\d+\.\d+\.\d+)?$/,
                type: 'asset/resource'
            }
        ]
    },

    plugins: [
        /**
         * Plugin: HtmlWebpackPlugin
         * Description: Simplifies creation of HTML files to serve your webpack bundles.
         * This is especially useful for webpack bundles that include a hash in the filename
         * which changes every compilation.
         *
         * See: https://github.com/ampedandwired/html-webpack-plugin
         */
        new htmlWebpackPlugin({
            template: path.resolve(`${settings.rootDir}/${settings.rootFile}`),
            filename: settings.rootFile,
            hash: true
        }),

        new AngularWebpackPlugin({
            tsconfig: `./examples/tsconfig.json`,
            compilerOptions: {
                paths: {
                    [`@progress/${settings.libraryName}`]: [ './src/main' ]
                }
            },
            jitMode: true
        }),

        new webpack.HotModuleReplacementPlugin(),

        /**
         * Async type and lint checking
         */
        // TODO: Investigate and re-enable
        // new forkTsCheckerWebpackPlugin({})
    ],
    optimization: {
        splitChunks: {
            chunks: 'all',
            name: 'vendors'
        }
    },

    // https://webpack.js.org/configuration/cache/#cachetype
    cache: {
        type: 'memory'
    }
});

module.exports = config;
