const fs = require('fs');
const path = require('path');
const header = require('gulp-header');
const replace = require('gulp-replace');
const uglify = require('gulp-uglify');
const ngPackagr = require('ng-packagr');
const pipeline = require('readable-stream').pipeline;
const { umdExportName } = require('@progress/kendo-common-tasks');

const packageConfigPath = './ng-package.json';
const encoding = 'utf-8';

const addBuildBundlesTask = (gulp, libraryName) => {
    const addLicense = () =>
        gulp.src([ './dist/**/*.js', './dist/**/*.ts' ])
            .pipe(header(licenseText()))
            .pipe(gulp.dest('./dist'));

    const stripSourcemaps = () =>
        gulp.src([ './dist/**/*.js' ])
            .pipe(replace(/\/\/# sourceMappingURL=.*/, ''))
            .pipe(gulp.dest('./dist'));

    const makePackage = () => {
        writePackageConfig(libraryName);

        return ngPackagr
            .ngPackagr()
            .forProject(packageConfigPath)
            .withTsConfig('tsconfig.json')
            .build()
            .then(() => {
                fs.unlinkSync(`./dist/bundles/${libraryName}.umd.js.map`);
                fs.unlinkSync(`./dist/fesm2015/${libraryName}.js.map`);
            });
    };

    const uglifyUmdBundle = () => pipeline(
        gulp.src(`./dist/bundles/*.js`),
        uglify(),
        gulp.dest('./dist/bundles')
    );

    gulp.task('build-bundles', gulp.series(
        makePackage,
        uglifyUmdBundle,
        addLicense,
        stripSourcemaps,
    ));
};

function licenseText() {
    return fs.readFileSync(path.join(__dirname, '..', 'license.txt'), { encoding: 'utf-8' });
}

function writePackageConfig(libraryName) {
    const meta = JSON.parse(fs.readFileSync('./package.json', { encoding }));

    let assets = [
        "LICENSE.md",
        "NOTICE.txt",
        "schematics"
    ];

    if (meta.assets) {
        const uniqueAssets = new Set([
            ...assets,
            ...meta.assets
        ]);
        assets = [ ...uniqueAssets ];
    }

    const dependencies = Object.keys(meta.dependencies).sort();
    const peerDependencies = Object.keys(meta.peerDependencies);
    const umdId = umdExportName(libraryName);
    const umdIdMap = [ ...dependencies, ...peerDependencies ]
        .filter(entry => entry.startsWith('@progress/kendo-') || entry.startsWith('@telerik/kendo-'))
        .sort()
        .reduce((map, name) => {
            const packageName = name.replace(/^@.+\//, '');
            map[name] = umdExportName(packageName);
            return map;
        }, {});


    const config = {
        dest: './dist',
        assets,
        allowedNonPeerDependencies: dependencies.filter(entry => entry.startsWith('@progress') || entry.startsWith('@telerik')),
        lib: {
            entryFile: './src/main.ts',
            flatModuleFile: libraryName,
            amdId: umdId,
            umdId: umdId,
            umdModuleIds: umdIdMap
        }
    };

    fs.writeFileSync(packageConfigPath, JSON.stringify(config, null, 2), { encoding });
}

module.exports = addBuildBundlesTask;
